/*
 *  Copyright 2001-2010 Mikhail Kryshen <mikhail@kryshen.net>
 *
 *  This file is part of Graph Game.
 *
 *  Graph Game is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Graph Game is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Graph Game.  If not, see <http://www.gnu.org/licenses/>.
 */
package kryshen.graphg;

import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.Enumeration;

class HumanPlayer extends Player 
    implements MouseListener, MouseMotionListener {
    
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    public void mouseReleased(MouseEvent e) {}
    public void mouseClicked(MouseEvent e) {}
    public void mouseDragged(MouseEvent e) {}

    public void mousePressed(MouseEvent e) {
	if (!myTurn | canvas.selectedEdge == null)
	    return;

	switch (player) {
	case SHORT:
	    canvas.selectedEdge.shorten();
	    break;
	case CUT:
	    graph.removeEdge(canvas.selectedEdge);
	}

	canvas.setSelectedEdge(null);

	//graph.setTagTo++;
	//int result = findShortestPath(graph.start, graph.finish, false);

	//Vector ps = graph.findShortestPaths(graph.start, graph.finish);
	//int result;
	//if(ps.size() == 0)
	//    done(DISCONNECTER);
	//else if (((GPath)ps.elementAt(0)).edges.size() == 0)
	//    done(CONNECTER);
	//else
	done();
    }
	//if (result < 0)
	//    done(DISCONNECTER);
	//else if(result == 0)
	//	done(CONNECTER);
	//else
	//    done(0);
	//}

    public void mouseMoved(MouseEvent e) {
 	if (!myTurn) return;
	
 	float kx = (canvas.getSize().width - canvas.border * 2) / (float)graph.width;
 	float ky = (canvas.getSize().height - canvas.border * 2) / (float)graph.height;
 	int x = Math.round((e.getX() - canvas.border) / kx);
 	int y = Math.round((e.getY() - canvas.border) / ky);
	Point c = new Point(x, y);
		
	Edge sEdge = null;
	double minDist = graph.width / 20;
	double dist;
 	Enumeration en = graph.edges.elements();
 	while (en.hasMoreElements()) {
 	    Edge link = (Edge)en.nextElement();
 	    //if (!link.connected) {
		Node a = link.node1;
		Node b = link.node2;
		double ab2 = getLineLength(a, b); ab2 = ab2 * ab2;
		if (ab2 != 0) {
		    double ac = getLineLength(a, c); 
		    double ac2 = ac * ac;
		    double bc = getLineLength(b, c);
		    double bc2 = bc * bc;
		    if (ac2 + bc2 <= ab2)
			dist = Math.sqrt(ac2 - ((ac2 + ab2 - bc2) * (ac2+ab2-bc2))/(4 * ab2));
		    else
			dist = Math.min(ac, bc);

		    if (dist <= minDist) {
			minDist = dist;
			sEdge = link;
		    }
		}
		//}
	}

// 	Edge oldSEdge = canvas.selectedEdge;

// 	if(canvas.selectedEdge == null) {
// 	    canvas.selectedEdge = sEdge;
// 	    canvas.repaint(sEdge);
// 	} else if(!canvas.selectedEdge.equals(sEdge)) {
// 	    canvas.selectedEdge = sEdge;
// 	    canvas.repaint(oldSEdge);
// 	    canvas.repaint(sEdge);
// 	}
	
	if (sEdge != null) {
	    if (!sEdge.shorted)
		canvas.setSelectedEdge(sEdge);
	    //System.err.println("*** " + sEdge.value);
	}


	//System.out.println(canvas.selectedEdge.node1.gnode.edges.size() +
	//		   "; " + 
	//		   canvas.selectedEdge.node2.gnode.edges.size());
    }

    public static double getLineLength(Point p1, Point p2) {
	int lx = p2.x - p1.x; lx = lx * lx;
	int ly = p2.y - p1.y; ly = ly * ly;
	return Math.sqrt(lx + ly);
    }

    public HumanPlayer(String name, int player, GraphCanvas gc) {
	super(name, player, gc);
	canvas.addMouseListener(this);
	canvas.addMouseMotionListener(this);
    }

    public synchronized void stop() {
	canvas.removeMouseListener(this);
	canvas.removeMouseMotionListener(this);
	super.stop();
    }    
}
