/*
 *  Copyright 2001-2010 Mikhail Kryshen <mikhail@kryshen.net>
 *
 *  This file is part of Graph Game.
 *
 *  Graph Game is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Graph Game is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Graph Game.  If not, see <http://www.gnu.org/licenses/>.
 */
package kryshen.graphg;

import java.awt.AWTEventMulticaster;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

public class NICButton extends Component
        implements MouseListener, MouseMotionListener, Runnable {

    Color backColor;
    Color textColor;
    Color borderColor;

    int borderWidth;    
    int shape;

    String text;
    private Image icon = null;

    private int y;
    protected boolean hovered = false;
    protected boolean pressed = false;

    private Thread tme = null;
    private int tmep = 101;
    private int atmep;

    private ActionListener al = null;
    private String actionCommand = null;

    private Dimension preferredSize = new Dimension(200, 30);

    private final Object lock = new Object();

    public NICButton(String caption, int width, int height) {
        backColor = new Color(240, 240, 240);
        textColor = new Color(0, 0, 255);
        borderColor = new Color(240, 0, 0);
        setFont(new Font("Helvetica", Font.PLAIN, 12));
        text = caption;
        shape = 0;
        borderWidth = 1;

        setSize(width, height);

        addMouseListener(this);
        addMouseMotionListener(this);
    }

    public NICButton(String caption, int width, int height, ActionListener l) {
        al = l;

        backColor = new Color(240, 240, 240);
        textColor = new Color(0, 0, 255);
        borderColor = new Color(240, 0, 0);
        setFont(new Font("Helvetica", Font.PLAIN, 12));
        text = caption;
        shape = 0;
        borderWidth = 1;

        setSize(width, height);

        addMouseListener(this);
        addMouseMotionListener(this);
    }

    public void addActionListener(ActionListener l) {
        al = AWTEventMulticaster.add(al, l);
    }

    public void removeActionListener(ActionListener l) {
        al = AWTEventMulticaster.remove(al, l);
    }

    public void setActionCommand(String actionCommand) {
        this.actionCommand = actionCommand;
    }

    void setIcon(Image icon) {
        this.icon = icon;
        repaint();
    }

//     private void createScaledIcon() {
// 	if (icon == null) {
// 	    scaledIcon = null;
// 	    return;
// 	}
// 	iconHeight = Math.min((int)(getSize().height * 0.6F), icon.getHeight(null));
// 	iconWidth = (int)(((float)iconHeight / icon.getHeight(null)) * icon.getWidth(null));
// 	System.err.println(iconWidth + " x " + iconHeight);
// 	scaledIcon = icon.getScaledInstance(iconWidth, iconHeight, Image.SCALE_SMOOTH);
// 	MediaTracker tracker = new MediaTracker(this);
// 	tracker.addImage(scaledIcon, 0);
// 	try {
// 	    tracker.waitForID(0);
// 	} catch (Exception e) {}
//     }
    public void setSize(int width, int height) {
        preferredSize = new Dimension(width, height);
        super.setSize(width, height);
        if (getSize().height <= getSize().width) {
            y = getSize().height / 2;
        } else {
            y = getSize().width / 2;
        }

        //synchronized (iconLock) {
        //    createScaledIcon();
        //}
        //repaint();
    }

    public Color makeLighter(Color clr, int rp, int gp, int bp) {
        int rp2 = clr.getRed() + rp;
        int gp2 = clr.getGreen() + gp;
        int bp2 = clr.getBlue() + bp;

        if (rp2 > 255) {
            rp2 = 255;
        }
        if (rp2 < 0) {
            rp2 = 0;
        }
        if (gp2 > 255) {
            gp2 = 255;
        }
        if (gp2 < 0) {
            gp2 = 0;
        }
        if (bp2 > 255) {
            bp2 = 255;
        }
        if (bp2 < 0) {
            bp2 = 0;
        }

        return new Color(rp2, gp2, bp2);
    }

    public void run() {
        while (tme == Thread.currentThread()) {
            try {
                Thread.sleep(40);
            } catch (Exception e) {
            }

            synchronized (lock) {
                tmep += atmep;
                if (tmep <= 0 || tmep > 100) {
                    tme = null;
                }
            }

            repaint();
        }
    }

    public void paint(Graphics g) {
        int width = getSize().width;
        int height = getSize().height;

        g.setFont(getFont());

        Color c = backColor;

        if (pressed) {
            c = makeLighter(backColor, -20, -20, -20);
        }

        g.setColor(c);
        g.fillRect(0, y, getSize().width - 1, getSize().height - y - 1);

        for (int i = 0; i <= y; i++) {
            g.setColor(new Color(clalcX(c.getRed(), y, i, true),
                    clalcX(c.getGreen(), y, i, true),
                    clalcX(c.getBlue(), y, i, true)));

            g.drawLine(0, y - i, getSize().width - 1, y - i);

            g.setColor(new Color(clalcX(c.getRed(), y, i, false),
                    clalcX(c.getGreen(), y, i, false),
                    clalcX(c.getBlue(), y, i, false)));

            g.drawLine(0, height - y + i - 1,
                    width - 1, height - y + i - 1);
        }

        switch (shape) {
            case 0:
                for (int i = 0; i <= y; i++) {
                    g.setColor(new Color(clalcX(c.getRed(), y, i, true),
                            clalcX(c.getGreen(), y, i, true),
                            clalcX(c.getBlue(), y, i, true)));

                    g.drawLine(y - i, y - i,
                            y - i, getSize().height - (y - i) - 1);

                    g.setColor(new Color(clalcX(c.getRed(), y, i, false),
                            clalcX(c.getGreen(), y, i, false),
                            clalcX(c.getBlue(), y, i, false)));

                    g.drawLine(width - y + i - 1, y - i,
                            width - y + i - 1,
                            height - (y - i) - 1);
                }
                break;

            case 1:
                Color cc1 = makeLighter(c, -100, -100, -100);
                Color cc2 = makeLighter(c, -50, -50, -50);

                if (pressed) {
                    g.setColor(cc2);
                } else {
                    g.setColor(cc1);
                }
                g.drawLine(width - 1, 0, width - 1, height - 1);

                if (pressed) {
                    g.setColor(cc1);
                } else {
                    g.setColor(cc2);
                }
                g.drawLine(0, 0, 0, height - 1);

                break;
        }

        int bl = pressed ? 1 : 0;
        int iw = 0;
        int trcw = 0;
        int trch = 0;
        int trcx = width / 2;
        int trcy = 0;
        int iconWidth = 0;
        int iconHeight = 0;
        FontMetrics fm = null;

        if (icon != null) {
            iconWidth = icon.getWidth(this);
            iconHeight = icon.getHeight(this);
            trcw = iconWidth;
            iw = iconWidth + 2;
            trcx = (width - trcw) / 2;
            trch = iconHeight;
        }

        if (text != null) {
            fm = g.getFontMetrics();
            g.setColor(textColor);
            trcw = fm.stringWidth(text) + iw;
            trch = Math.max(fm.getHeight(), trch);
            trcx = (width - trcw) / 2;
            trcy = (height - fm.getHeight()) / 2;
        }

        if (icon != null) {
            int iy = (height - iconHeight) / 2;
            g.drawImage(icon, trcx + bl, iy + bl, this);
        }

        if (text != null) {
            g.drawString(text, trcx + iw + bl,
                    trcy + bl + fm.getAscent());
        }

        if (tmep <= 100) {
            trcy = (height - trch) / 2;

            int tmx, tmy;
            synchronized (lock) {
                tmx = (int) Math.round(tmep / 100F * trcx);
                tmy = (int) Math.round(tmep / 100F * trcy);
            }
            for (int i = 1; i < borderWidth + 1; i++) {
                g.setColor(new Color(100, 100, 100));
                g.drawRect(trcx - i + bl - tmx,
                        trcy - i + bl - tmy,
                        trcw - 1 + 2 * i + 2 * tmx,
                        trch - 1 + 2 * i + 2 * tmy);
                g.setColor(borderColor);
                g.drawRect(trcx - i - tmx,
                        trcy - i - tmy,
                        trcw - 1 + 2 * i + 2 * tmx,
                        trch - 1 + 2 * i + 2 * tmy);
            }
        }


//  	if (me) {
//  	    g.setColor(borderColor);
//  	    for (int i=0; i < borderWidth; i++)
//  		g.drawRect(i, i, getSize().width - 1 - 2 * i, 
//  			   getSize().height - 1 - 2 * i);
//  	}
    }

    private int clalcX(int clr, int y, int i, boolean l) {
        int clr2 = clr - 80;
        if (l) {
            clr2 = clr2 + (pressed ? 40 : -40);
        }
        if (clr2 < 0) {
            clr2 = 0;
        } else if (clr2 > 255) {
            clr2 = 255;
        }
        double x = i * (Math.sqrt((double) clr2) / (double) y);
        return clr - (int) (x * x);
    }

    public void mouseDragged(MouseEvent evt) {
        if (evt.getX() > 0 && evt.getX() < getSize().width - 1 &&
                evt.getY() > 0 && evt.getY() < getSize().height - 1) {
            if (pressed && hovered) {
                return;
            }
            pressed = true;
            hovered = true;
        } else {
            if (!pressed && !hovered) {
                return;
            }
            pressed = false;
            hovered = false;
        }
        repaint();
    }

    public void mouseEntered(MouseEvent evt) {
        hovered = true;
        repaint();

        synchronized (lock) {
            atmep = -10;
            if (tme == null) {
                tme = new Thread(this);
                tmep = 100;
                tme.start();
            }
        }
    }

    public void mouseExited(MouseEvent evt) {
        hovered = false;
        repaint();

        synchronized (lock) {
            atmep = 20;
            if (tme == null) {
                tme = new Thread(this);
                tme.start();
            }
        }
    }

    public void mousePressed(MouseEvent evt) {
        pressed = true;
        repaint();
    }

    public void mouseReleased(MouseEvent evt) {
        if (!pressed) {
            return;
        }
        pressed = false;
        repaint();

        if (hovered && al != null) {
            al.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED,
                    (actionCommand == null) ? text : actionCommand));
        }
    }

    public void mouseClicked(MouseEvent evt) {
    }

    public void mouseMoved(MouseEvent e) {
    }

    public boolean isOpaque() {
        return true;
    }

    public Dimension getPreferredSize() {
        return preferredSize;
    }

    public void setText(String text) {
        this.text = text;
        repaint();
    }
}
